use std::sync::{Arc, Mutex};
use actix_web::{App, HttpServer, web};
use actix_web::web::Data;
use dotenv_codegen::dotenv;
use backend::State;
use tokio_postgres::NoTls;
use backend::storages::postgres::car::PostgresCarRepository;
use backend::storages::postgres::car_station::PostgresCarStationRepository;
use backend::storages::postgres::client::PostgresClientRepository;
use backend::storages::postgres::rent::PostgresRentRepository;
use backend::storages::postgres::owner::PostgresOwnerRepository;
use backend::endpoints::car_station::*;
use backend::endpoints::car::*;
use backend::endpoints::client::*;
use backend::endpoints::rent::*;
use backend::endpoints::owner::*;

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    let (client, connection) = tokio_postgres::connect(
        &format!("host={} user={} password={} dbname={}", dotenv!("HOST"), dotenv!("USER"), dotenv!("PASSWORD"), dotenv!("DBNAME")),
        NoTls
    ).await.unwrap();

    tokio::spawn(async move {
        if let Err(e) = connection.await {
            eprintln!("connection error: {}", e);
        }
    });

    let client = Arc::new(client);

    let car_repository = PostgresCarRepository { connection: Arc::clone(&client) };

    let car_station_repository = PostgresCarStationRepository { connection: Arc::clone(&client) };

    let client_repository = PostgresClientRepository { connection: Arc::clone(&client) };

    let owner_repository = PostgresOwnerRepository { connection: Arc::clone(&client) };

    let rent_repository = PostgresRentRepository { connection: Arc::clone(&client) };

    let state = Data::new(Mutex::new(State {
        car_repository: Box::new(car_repository),
        car_station_repository: Box::new(car_station_repository),
        client_repository: Box::new(client_repository),
        owner_repository: Box::new(owner_repository),
        rent_repository: Box::new(rent_repository)
    }));

    HttpServer::new(move || {
        App::new()
            .app_data(Data::clone(&state))
            .service(
                web::scope("/cars")
                    .service(get_cars)
                    .service(get_car)
                    .service(create_car)
                    .service(update_car)
                    .service(delete_car)
            )
            .service(
                web::scope("/clients")
                    .service(get_clients)
                    .service(get_client)
                    .service(create_client)
                    .service(update_client)
                    .service(delete_client)
            )
            .service(
                web::scope("/owners")
                    .service(get_owners)
                    .service(get_owner)
                    .service(create_owner)
                    .service(update_owner)
                    .service(delete_owner)
            )
            .service(
                web::scope("/car_stations")
                    .service(get_car_stations)
                    .service(get_car_station)
                    .service(create_car_station)
                    .service(update_car_station)
                    .service(delete_car_station)
            )
            .service(
                web::scope("/rents")
                    .service(get_rents)
                    .service(get_rent)
                    .service(create_rent)
                    .service(update_rent)
                    .service(delete_rent)
            )
    })
        .bind("127.0.0.1:8080")?
        .run()
        .await
}